----------------------------------------------------------------------------------
--
-- Copyright CERN 2011.
-- 
-- This documentation describes Open Hardware and is licensed under the
-- CERN OHL v. 1.1.
--
-- You may redistribute and modify this documentation under the terms of the CERN
-- OHL v.1.1. (http://ohwr.org/cernohl).
--
-- This documentation is distributed WITHOUT
-- ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING OF MERCHANTABILITY, SATISFACTORY
-- QUALITY AND FITNESS FOR A PARTICULAR PURPOSE. Please see the CERN OHL v.1.1 for
-- applicable conditions.
--
----------------------------------------------------------------------------------
--
-- Company:        CERN, BE-CO-HT 
-- Engineer:       Carlos Gil Soriano
-- 
-- Create Date:    12:00:00 15-06-2012 
-- Design Name:
-- Module Name:    m25p32_pkg
-- Project Name:   Level Conversion Circuits 
-- Target Devices: Spartan 6
-- Tool versions:  Xilinx ISE 13.4
-- Description:    This package holds the definition of the register used along
--                 the m25p32 core. It acts the logic that glues together the
--                 following modules:
--                   - m25p32_core.vhd
--                   - m25p32_regs.vhd
--
-- Dependencies:   spi_master_pkg.vhd (SPI master multifield) 
--
-- Revision: 0.1
--   0.01          + File Created
--                 
-- Additional Comments: enjoy it! ;) 
--
----------------------------------------------------------------------------------




library IEEE;
library work;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.ALL;
use work.spi_master_pkg.ALL;

package m25p32_pkg is

   constant CPOL                : STD_LOGIC := '0';
   constant CPHA                : STD_LOGIC := '1';

   --! Constant only used in simulation
   constant c_WISHBONE_PERIOD   : TIME      := 50 ns;  --! Working at 20MHz 

   --! General constants
   constant c_SECTOR_BITS           : NATURAL := 6; -- 64 sectors
   constant c_PAGES_PER_SECTOR_BITS : NATURAL := 8; -- 256 pages/sector
   constant c_BYTES_PER_PAGE_BITS   : NATURAL := 8; -- 256 bytes = 64 words
   constant c_WORDS_PER_PAGE_BITS   : NATURAL := c_BYTES_PER_PAGE_BITS - 2;
   constant c_ADDR_PADDING          : NATURAL := 24
                                                 - c_SECTOR_BITS
                                                 - c_PAGES_PER_SECTOR_BITS
                                                 - c_BYTES_PER_PAGE_BITS; 

   constant c_PAGE_SIZE            : NATURAL := 256;   --! Bytes in a page

   --! Mapping of the registers
   constant c_WB_ADDR_PADDING : STD_LOGIC_VECTOR(c_WORDS_PER_PAGE_BITS - 1
                                  downto 2) := (others => 0);
   ----------------------------------------
   --  Memory Map
   ----------------------------------------
   --  WBits
   ----------------------------------------
   --       [POSITION]            [CONTENT]
   --    0[WBits-2]00                 FMI 
   --    0[WBits-2]01                  SR
   --    0[WBits-2]10           READ_DATA
   --    1[WBits-2]00             WR_DATA
   --             ...                 ...                               
   --    1[WBits-2]00             WR_DATA
   ----------------------------------------
   constant c_FMI_addr        :STD_LOGIC_VECTOR (c_WORDS_PER_PAGE_BITS
                                  downto 2) :=
                                  '0' & c_WB_ADDR_PADDING & "00"; 
   constant c_SR_m25p32_addr  : STD_LOGIC_VECTOR (c_WORDS_PER_PAGE_BITS
                                  downto 0) :=
                                  '0' & c_WB_ADDR_PADDING & "01";
   constant c_DATA_READ_addr  : STD_LOGIC_VECTOR (c_WORDS_PER_PAGE_BITS
                                  downto 0) :=
                                  '0' & c_WB_ADDR_PADDING & "10";
   constant c_DATA_WRITE_addr : STD_LOGIC_VECTOR (c_WORDS_PER_PAGE_BITS
                                  downto 0) :=
                                  '1' & c_WB_ADDR_PADDING & "00";

   --! Attribute length for every register
   attribute a_length : NATURAL;

   ----------------------------------------
   --  Flash Memory Instruction
   ----------------------------------------
   --  16 bits
   --  Wishbone access: Write-read
   ----------------------------------------
   --  BIT        NAME      DESCRIPTION 
   --   0          OPR       OPeration Request (WR)
   --   1          OPA       OPeration Attended (R)
   --   2          OPF       Operation Finished (R)
   --   4-3        OP        OPeration         (WR)
   --   12-5       PG        PaGe number       (WR)
   --   18-13      SCT       SeCTor number     (WR)
   ----------------------------------------
   --  List of operations:
   --  0b000       x         Reserved
   --  0b001       RDID      ReaD IDentification
   --  0b010       RDSR      ReaD Status Register 
   --  0b011       WRSR      WRite Status Register
   --  0b100       READ      ReaD Page
   --  0b101       PP        Page Program
   --  0b110       SE        Sector Erase
   --  0b111       BE        Block Erase
   ----------------------------------------
   type t_operations is (x, READ, PP, RDSR,
                         WRSR, RDID, SE, BE); 

   type r_FMI is
      record
         OPR   : STD_LOGIC;
         OPA   : STD_LOGIC;
         OPF   : STD_LOGIC;
         OP    : t_operations;
         PG    : STD_LOGIC_VECTOR(c_PAGES_PER_SECTOR_BITS - 1 + 5 downto  5);
         SCT   : STD_LOGIC_VECTOR(c_SECTOR_BITS + c_PAGES_PER_SECTOR_BITS
                                                          - 1 + 5 downto
                                                c_PAGES_PER_SECTOR_BITS + 5);
         x     : STD_LOGIC_VECTOR(23 downto c_SECTOR_BITS + c_PAGES_PER_SECTOR_BITS
                                                          + 5);
      end record;

   attribute a_length of r_FMI      : type is 24;


   ----------------------------------------
   -- Status Register in m25p32
   ----------------------------------------
   -- 8 bits
   -- Wishbone access: Write-read
   ----------------------------------------
   --  BIT        NAME      DESCRIPTION
   --    0         WIP       Write In Progress 
   --    1         WEL       Write Enable Latch
   --    2         BP0       Block Protect bit 0 
   --    3         BP1       Block Protect bit 1
   --    4         BP2       Block Protect bit 2
   --  6-5         x         Reserved
   --    7         SRWD      Status Register Write Disabled
   ----------------------------------------
   type r_SR_m25p32 is
      record
         WIP  : STD_LOGIC;
         WEL  : STD_LOGIC;
         BP0  : STD_LOGIC;
         BP1  : STD_LOGIC;
         BP2  : STD_LOGIC;
         x    : STD_LOGIC_VECTOR(6  downto 5);
         SRWD : STD_LOGIC; 
      end record;

   attribute a_length of r_SR_m25p32 : type is 8;


   type r_word_align is
      record
         word_bit : STD_LOGIC_VECTOR(31 downto 0);
      end record;

   type r_page is
      record
         word : r_word(c_WORDS_PER_PAGE_BITS - 1 downto 0);
      end record;

   ---------------------------------------------------------------------------
   --! Default values for the registers
   ---------------------------------------------------------------------------

   --! Default: FMI no operation set by default
   constant c_FMI_default  : r_FMI := (OPR     => '0',
                                       OPA     => '0',
                                       OPF     => '0',
                                       OP      => x,
                                       PG      => (others => '0'),
                                       SCT     => (others => '0'),
                                       x       => (others => '0'));

   --! Default: SR_m25p32 no protection in the memory by default
   constant c_SR_m25p32_default : r_SR_m25p32 := (WIP  => '0', 
                                                  WEL  => '0', 
                                                  BP0  => '0', 
                                                  BP1  => '0', 
                                                  BP2  => '0', 
                                                  x    => (others => '0'), 
                                                  SRWD => '0');

   ------------------------------------------------------------------------------
   --  m25p32 m25p32 m25p32 m25p32 m25p32 SPI Instruction set
   ------------------------------------------------------------------------------
   --  INSTRUCTION        DESCRIPTION                ADDRESS         DATA
   --                                                 BYTES          BYTES
   --    WREN              WRite ENable                 0             0
   --    WRDI              WRite DIsable                0             0
   --    RDID              ReaD IDentification          0             3
   --    RDSR              ReaD Status Register         0             1
   --    WRSR              WRite Status Register        0             1
   --    READ              READ data bytes              3             4
   --    PP                Page Program                 3             256
   --    SE                Sector Erase                 3             0
   --    BE                Bulk Erase                   0             0
   -------------------------------------------------------------------------------
   constant c_WREN_inst      : STD_LOGIC_VECTOR(7  downto 0) := X"06";
   constant c_WRDI_inst      : STD_LOGIC_VECTOR(7  downto 0) := X"04";
   constant c_RDID_inst      : STD_LOGIC_VECTOR(7  downto 0) := X"9F";
   constant c_RDSR_inst      : STD_LOGIC_VECTOR(7  downto 0) := X"05";
   constant c_WRSR_inst      : STD_LOGIC_VECTOR(7  downto 0) := X"01";
   constant c_READ_inst      : STD_LOGIC_VECTOR(7  downto 0) := X"03";
   --   constant c_FAST_READ_inst : STD_LOGIC_VECTOR(7  downto 0) := X"0B";
   constant c_PP_inst        : STD_LOGIC_VECTOR(7  downto 0) := X"02";
   constant c_SE_inst        : STD_LOGIC_VECTOR(7  downto 0) := X"D8";
   constant c_BE_inst        : STD_LOGIC_VECTOR(7  downto 0) := X"C7";

   constant c_SPI0_IDLE  : r_SPI0 := (CPOL  => c_CPOL,
                                      CPHA  => c_CPHA,
                                      BREAD => (others => '0'),
                                      BDATA => (others => '0'),
                                      BADDR => (others => '0'),
                                      BINST => (others => '0');

      constant c_SPI1_IDLE : r_SPI1 := (PUSH_DATA => '0', PUSH_ADDR => '0',
                                        PUSH_INST => '0', x => (others  => '0'),
                                        READ_MISO => '0',
                                        SEND_DATA => '0',
                                        SEND_ADDR => '0',
                                        SEND_INST => '0',
                                        SEND_OP   => '0',
                                        y         => (others  => '0'),
                                        CLK_DIV   => c_CLK_DIV,
                                        z         => (others  => '0'));


   --! WREN instruction. Configuration for SPI0 and SPI1 registers.
   constant c_SPI0_WREN  : r_SPI0 := (CPOL  => c_CPOL,
                                      CPHA  => c_CPHA,
                                      BREAD => (others => '0'),
                                      BDATA => (others => '0'),
                                      BADDR => (others => '0'),
                                      BINST => to_unsigned( 1 , 9));

      constant c_SPI1_WREN : r_SPI1 := (PUSH_DATA => '0', PUSH_ADDR => '0',
                                        PUSH_INST => '1', x => (others  => '0'),
                                        READ_MISO => '0',
                                        SEND_DATA => '0',
                                        SEND_ADDR => '0',
                                        SEND_INST => '1',
                                        SEND_OP   => '1',
                                        y         => (others  => '0'),
                                        CLK_DIV   => c_CLK_DIV,
                                        z         => (others  => '0'));


   --! WRDI instruction. Configuration for SPI0 and SPI1 registers.
   constant c_SPI0_WRDI  : r_SPI0 := (CPOL  => c_CPOL,
                                      CPHA  => c_CPHA,
                                      BREAD => (others => '0'),
                                      BDATA => (others => '0'),
                                      BADDR => (others => '0'),
                                      BINST => to_unsigned( 1, 9));

      constant c_SPI1_WRDI : r_SPI1 := (PUSH_DATA => '0', PUSH_ADDR => '0',
                                        PUSH_INST => '1', x => (others  => '0'),
                                        READ_MISO => '0',
                                        SEND_DATA => '0',
                                        SEND_ADDR => '0',
                                        SEND_INST => '1',
                                        SEND_OP   => '1',
                                        y         => (others  => '0'),
                                        CLK_DIV   => c_CLK_DIV,
                                        z         => (others  => '0'));


   --! RDID instruction. Configuration for SPI0 and SPI1 registers.
   constant c_SPI0_RDID  : r_SPI0 := (CPOL  => c_CPOL,
                                      CPHA  => c_CPHA,
                                      BREAD => to_unsigned( 3, 4),
                                      BDATA => (others => '0'),
                                      BADDR => (others => '0'),
                                      BINST => to_unsigned( 1, 9));

      constant c_SPI1_RDID : r_SPI1 := (PUSH_DATA => '0', PUSH_ADDR => '0',
                                        PUSH_INST => '1', x => (others  => '0'),
                                        READ_MISO => '1',
                                        SEND_DATA => '0',
                                        SEND_ADDR => '0',
                                        SEND_INST => '1',
                                        SEND_OP   => '1',
                                        y         => (others  => '0'),
                                        CLK_DIV   => c_CLK_DIV,
                                        z         => (others  => '0'));


   --! SRWD instruction. Configuration for SPI0 and SPI1 registers.
   constant c_SPI0_RDSR  : r_SPI0 := (CPOL  => c_CPOL,
                                      CPHA  => c_CPHA,
                                      BREAD => to_unsigned( 1, 4),
                                      BDATA => (others => '0'),
                                      BADDR => (others => '0'),
                                      BINST => to_unsigned( 1, 9));

      constant c_SPI1_RDSR : r_SPI1 := (PUSH_DATA => '0', PUSH_ADDR => '0',
                                        PUSH_INST => '1', x => (others  => '0'),
                                        READ_MISO => '1',
                                        SEND_DATA => '0',
                                        SEND_ADDR => '0',
                                        SEND_INST => '1',
                                        SEND_OP   => '1',
                                        y         => (others  => '0'),
                                        CLK_DIV   => c_CLK_DIV,
                                        z         => (others  => '0'));


   --! WRSR instruction. Configuration for SPI0 and SPI1 registers.
   constant c_SPI0_WRSR  : r_SPI0 := (CPOL  => c_CPOL,
                                      CPHA  => c_CPHA,
                                      BREAD => (others => '0'),
                                      BDATA => to_unsigned( 1 , 9),
                                      BADDR => (others => '0'),
                                      BINST => to_unsigned( 1 , 9));

      constant c_SPI1_WRSR : r_SPI1 := (PUSH_DATA => '1', PUSH_ADDR => '0',
                                        PUSH_INST => '1', x => (others  => '0'),
                                        READ_MISO => '0',
                                        SEND_DATA => '1',
                                        SEND_ADDR => '0',
                                        SEND_INST => '1',
                                        SEND_OP   => '1',
                                        y         => (others  => '0'),
                                        CLK_DIV   => c_CLK_DIV,
                                        z         => (others  => '0'));


   --! READ instruction. Configuration for SPI0 and SPI1 registers.
   constant c_SPI0_READ  : r_SPI0 := (CPOL  => c_CPOL,
                                      CPHA  => c_CPHA,
                                      --! Aribtrary value of 4.
                                      --! According to manual it reads the
                                      --! whole memory.
                                      BREAD => to_unsigned( 4, 4),
                                      BDATA => to_unsigned( 1 , 9),
                                      BADDR => (others => '0'),
                                      BINST => to_unsigned( 1 , 9));

      constant c_SPI1_READ : r_SPI1 := (PUSH_DATA => '1', PUSH_ADDR => '0',
                                        PUSH_INST => '1', x => (others  => '0'),
                                        READ_MISO => '1',
                                        SEND_DATA => '1',
                                        SEND_ADDR => '0',
                                        SEND_INST => '1',
                                        SEND_OP   => '1',
                                        y         => (others  => '0'),
                                        CLK_DIV   => c_CLK_DIV,
                                        z         => (others  => '0'));


   --! PP instruction. Configuration for SPI0 and SPI1 registers.
   constant c_SPI0_PP    : r_SPI0 := (CPOL  => c_CPOL,
                                      CPHA  => c_CPHA,
                                      BREAD => (others => '0'),
                                      BDATA => to_unsigned( 256 , 9),
                                      BADDR => to_unsigned(   3 , 9),
                                      BINST => to_unsigned(   1 , 9));

      constant c_SPI1_PP : r_SPI1 := (PUSH_DATA => '0', PUSH_ADDR => '0',
                                      PUSH_INST => '0', x => (others  => '0'),
                                      READ_MISO => '0',
                                      SEND_DATA => '1',
                                      SEND_ADDR => '1',
                                      SEND_INST => '1',
                                      SEND_OP   => '1',
                                      y         => (others  => '0'),
                                      CLK_DIV   => c_CLK_DIV,
                                      z         => (others  => '0'));


   --! SE instruction. Configuration for SPI0 and SPI1 registers.
   constant c_SPI0_SE    : r_SPI0 := (CPOL  => c_CPOL,
                                      CPHA  => c_CPHA,
                                      BREAD => (others => '0'),
                                      BDATA => (others => '0'),
                                      BADDR => to_unsigned( 3 , 9), 
                                      BINST => to_unsigned( 1 , 9));

      constant c_SPI1_SE : r_SPI1 := (PUSH_DATA => '0', PUSH_ADDR => '0',
                                      PUSH_INST => '0', x => (others  => '0'),
                                      READ_MISO => '0',
                                      SEND_DATA => '0',
                                      SEND_ADDR => '1',
                                      SEND_INST => '1',
                                      SEND_OP   => '1',
                                      y         => (others  => '0'),
                                      CLK_DIV   => c_CLK_DIV,
                                      z         => (others  => '0'));

   --! BE instruction. Configuration for SPI0 and SPI1 registers.
   constant c_SPI0_BE    : r_SPI0 := (CPOL  => c_CPOL,
                                      CPHA  => c_CPHA,
                                      BREAD => (others => '0'),
                                      BDATA => (others => '0'),
                                      BADDR => (others => '0'), 
                                      BINST => to_unsigned( 1 , 9));

      constant c_SPI1_BE : r_SPI1 := (PUSH_DATA => '0', PUSH_ADDR => '0',
                                      PUSH_INST => '0', x => (others  => '0'),
                                      READ_MISO => '0',
                                      SEND_DATA => '0',
                                      SEND_ADDR => '0',
                                      SEND_INST => '1',
                                      SEND_OP   => '1',
                                      y         => (others  => '0'),
                                      CLK_DIV   => c_CLK_DIV,
                                      z         => (others  => '0'));


   --! Included here to be less verbose in m25p32_core.vhd
   -- c_CLK_I_PERIOD can be found in spi_master_pkg.vhd
   component spi_master_core is
      generic(
        g_INST_LENGTH  : NATURAL := c_INST_LENGTH;
        g_ADDR_LENGTH  : NATURAL := c_ADDR_LENGTH;
        g_DATA_LENGTH  : NATURAL := c_DATA_LENGTH;
        g_READ_LENGTH  : NATURAL := c_READ_LENGTH;
        g_CLK_I_PERIOD : NATURAL := c_CLK_I_PERIOD);
      port(
        rst_i        : in  STD_LOGIC;
        clk_i        : in  STD_LOGIC;

        inst_i       : in  STD_LOGIC_VECTOR (8*g_INST_LENGTH - 1 downto 0);
        addr_i       : in  STD_LOGIC_VECTOR (8*g_ADDR_LENGTH - 1 downto 0);
        data_i       : in  STD_LOGIC_VECTOR (8*g_DATA_LENGTH - 1 downto 0);

        data_o       : out STD_LOGIC_VECTOR (8*g_READ_LENGTH - 1 downto 0);

        SPI0_i       : in  STD_LOGIC_VECTOR (31 downto 0);
        SPI1_i       : in  STD_LOGIC_VECTOR (31 downto 0);
        SPI2_o       : out STD_LOGIC_VECTOR (15 downto 0);
        SPI3_o       : out STD_LOGIC_VECTOR (31 downto 0);

        spi_mosi_o   : out STD_LOGIC;
        spi_miso_i   : in  STD_LOGIC;
        spi_clk_o    : out STD_LOGIC;
        spi_cs_n_o   : out STD_LOGIC);
   end component;

   ---------------------------------------------------------------------------
   --! Translation functions back and forth STD_LOGIC_VECTOR
   ---------------------------------------------------------------------------
   function f_STD_LOGIC_VECTOR  (r_register : r_FMI)  return STD_LOGIC_VECTOR;
   function f_STD_LOGIC_VECTOR  (r_register : r_SR_m25p32)
                                                      return STD_LOGIC_VECTOR;
   function f_STD_LOGIC_VECTOR  (word       : r_word)
                                                      return STD_LOGIC_VECTOR;
   function f_STD_LOGIC_VECTOR  (page       : r_page)
                                                      return STD_LOGIC_VECTOR;

   function f_FMI       (signal r_register : in STD_LOGIC_VECTOR(31 downto 0))
                                                                return r_FMI;
   function f_SR_m25p32 (signal r_register : in STD_LOGIC_VECTOR(7  downto 0))
                                                           return r_SR_m25p32;
   function f_word (signal page : in r_page;
                    signal pos  : in UNSIGNED(c_WORDS_PER_PAGE_BITS -1 downto 0))
                                                                return r_word;
   function f_page (signal page : in STD_LOGIC_VECTOR(c_PAGE_SIZE*8 -1 downto 0))
                                                                return r_page;


end m25p32_pkg;

package body m25p32_pkg is


   -----------------------------------------------------------------------------
   --! @brief CTRL record type translation to STD_LOGIC_VECTOR
   --! @param r_register r_CTRL record type to be translated
   -----------------------------------------------------------------------------
   function f_STD_LOGIC_VECTOR  (r_register : r_CTRL) return STD_LOGIC_VECTOR is
   begin
      return ( r_register.x                      &
               std_logic_vector(r_register.CNTA) &
               r_register.PUSH_D                 &
               r_register.PUSH_A                 &
               r_register.PUSH_I                 &
               r_register.CBSF                   &
               r_register.ARBF                   &
               r_register.AWBF                   &
               r_register.OPF                    &
               r_register.OPA                    &);
   end f_STD_LOGIC_VECTOR;

   -----------------------------------------------------------------------------
   --! @brief Translation from STD_LOGIC_VECTOR to r_CTRL record
   --! @param r_register STD_LOGIC_VECTOR to be translated
   -----------------------------------------------------------------------------
   function f_CTRL  (signal r_register : in STD_LOGIC_VECTOR(23 downto 0))
                                                                return r_CTRL is
      variable  v_CTRL : r_CTRL;
   begin
      v_CTRL.OPA     := r_register(0);
      v_CTRL.OPF     := r_register(1);
      v_CTRL.AWBF    := r_register(  c_NUMBER_OF_BUFFERS - 1  downto 0);
      v_CTRL.ARBF    := r_register(2*c_NUMBER_OF_BUFFERS - 1  downto
                                                            c_NUMBER_OF_BUFFERS );
      v_CTRL.CBSF    := r_register(3*c_NUMBER_OF_BUFFERS - 1  downto 
                                                          2*c_NUMBER_OF_BUFFERS );
      v_CTRL.PUSH_I  := r_register(6);
      v_CTRL.PUSH_A  := r_register(7);
      v_CTRL.PUSH_D  := r_register(8);
      v_CTRL.CNTA    := unsigned(r_register(18 downto 11));
      v_CTRL.x       := r_register(23 downto 19);
      return v_CTRL; 
   end f_CTRL;

   -----------------------------------------------------------------------------
   --! @brief FMI record type translation to STD_LOGIC_VECTOR
   --! @param r_register r_FMI record type to be translated
   -----------------------------------------------------------------------------
   function f_STD_LOGIC_VECTOR  (r_register : r_FMI) return STD_LOGIC_VECTOR is
   begin
      return (r_register.x    &
              r_register.SCT  & r_register.PG   &
              r_register.OP   &
              r_register.OPF  & r_register.OPA &
              r_register.OPR); 
   end f_STD_LOGIC_VECTOR; 

   -----------------------------------------------------------------------------
   --! @brief Translation from STD_LOGIC_VECTOR to r_FMI record
   --! @param r_register STD_LOGIC_VECTOR to be translated
   -----------------------------------------------------------------------------
   function f_FMI(signal r_register : in STD_LOGIC_VECTOR(23  downto 0))
                                                                return r_FMI is
      variable  v_FMI : r_FMI;
   begin
      v_FMI.OPR  := r_register(0);
      v_FMI.OPA  := r_register(1);
      v_FMI.OPF  := r_register(2);
      v_FMI.OP   := r_register(4  downto 3);
      v_FMI.PG   := r_register(r_FMI.PG'high  downto r_FMI.PG'low);
      v_FMI.SCT  := r_register(r_FMI.SCT'high downto r_FMI.SCT'low);
      v_FMI.x    := r_register(23 downto r_FMI.SCT'high + 1);
      return v_FMI; 
   end f_FMI;

   -----------------------------------------------------------------------------
   --! @brief SR_m25p32 record type translation to STD_LOGIC_VECTOR
   --! @param r_register r_SR_m25p32 record type to be translated
   -----------------------------------------------------------------------------
   function f_STD_LOGIC_VECTOR  (r_register : r_SR_m25p32)
                                                      return STD_LOGIC_VECTOR is
   begin
      return ( r_register.SRWD & std_logic_vector(r_register.x) & 
               r_register.BP2  & r_register.BP1                 & 
               r_register.BP0  & r_register.WEL &
               r_register.WIP );
   end f_STD_LOGIC_VECTOR;

   -----------------------------------------------------------------------------
   --! @brief Translation from STD_LOGIC_VECTOR to r_SR_m25p32 record
   --! @param r_register STD_LOGIC_VECTOR to be translated
   -----------------------------------------------------------------------------
   function f_SR_m25p32 (signal r_register : in STD_LOGIC_VECTOR(7  downto 0))
   return r_SR_m25p32 is
      variable v_SR_m25p32 : r_SR_m25p32;
   begin
      v_SR_m25p32.WIP   := r_register(0);
      v_SR_m25p32.WEL   := r_register(1);
      v_SR_m25p32.BP0   := r_register(2);
      v_SR_m25p32.BP1   := r_register(3);
      v_SR_m25p32.BP2   := r_register(4);
      v_SR_m25p32.x     := r_register(6 downto 5);
      v_SR_m25p32.SRWD  := r_register(7);
      return v_SR_m25p32;
   end f_SR_m25p32;

   -----------------------------------------------------------------------------
   --! @brief r_word record type translation to STD_LOGIC_VECTOR
   --! @param word r_word record type to be translated
   -----------------------------------------------------------------------------
   function f_STD_LOGIC_VECTOR  (word       : r_word) return STD_LOGIC_VECTOR is
   begin
      return word.word_bit;
   end function;

   -----------------------------------------------------------------------------
   --! @brief Translation from STD_LOGIC_VECTOR to f_word record
   --! @param page Page to retrieve the word from.
   --! @param pos  Position of the word to be retrieved.
   -----------------------------------------------------------------------------
   function f_word (signal page : in r_page;
                    signal pos  : in UNSIGNED(c_WORDS_PER_PAGE_BITS -1 downto 0))
                   return r_word is
   begin
      return page.word(to_integer(pos));
   end function f_word;

   -----------------------------------------------------------------------------
   --! @brief r_page record type translation to STD_LOGIC_VECTOR
   --! @param page r_page record type to be translated
   -----------------------------------------------------------------------------
   function f_STD_LOGIC_VECTOR  (page       : r_page) return STD_LOGIC_VECTOR is
      variable v_page_slv : STD_LOGIC_VECTOR(c_PAGE_SIZE/4 - 1 downto 0);
      variable v_word_tmp : r_word;
   begin
      for i in 0 to c_PAGE_SIZE/4 - 1  loop
         v_word_tmp := page.word(i);
         v_page_slv(32*(i+1) - 1 downto 32*i) := f_STD_LOGIC_VECTOR(v_word_tmp);
      end loop;
      return v_page_slv;
   end function f_STD_LOGIC_VECTOR;

   -----------------------------------------------------------------------------
   --! @brief Translation from STD_LOGIC_VECTOR to r_page record
   --! @param page STD_LOGIC_VECTOR to be translated
   -----------------------------------------------------------------------------
   function f_page (signal page : in STD_LOGIC_VECTOR(c_PAGE_SIZE*8 -1 downto 0))
                   return r_page is
      variable v_page : r_page;
   begin
      for i in 0 to c_PAGE_SIZE/4 - 1 loop
         v_page.word(i) := page(32*(i+1) - 1 downto 32*i);
      end loop;
      return v_page;
   end function f_page;

end m25p32_pkg;
